import requests
import datetime
from loguru import logger
import os


def get_sign(timestamp: str) -> str:
    """获取签名
    签名算法说明
    签名算法选择：HmacSHA256

    签名密钥：secretKey，密钥和企业ID相对应，由接口提供方统一提供

    签名内容：corporationId + timestamp

    签名结果编码：base64
    """
    import hmac
    import hashlib
    import base64

    message = f"{os.getenv('CORPORATION_ID')}{timestamp}".encode("utf-8")
    secret = os.getenv("SECRET_KEY").encode("utf-8")
    hash = hmac.new(secret, message, hashlib.sha256)
    sign = base64.b64encode(hash.digest()).decode("utf-8")
    return sign


class base:

    def __init__(self, debug=False):
        self.debug = debug
        self.API_DOMAIN = os.getenv("API_DOMAIN")
        self.APP_KEY = os.getenv("APP_KEY")
        self.CORPORATION_ID = os.getenv("CORPORATION_ID")
        self.BIZ_ID = os.getenv("BIZ_ID")
        self.SECRET_KEY = os.getenv("SECRET_KEY")
        self.REPORT_DOMAIN = os.getenv("REPORT_DOMAIN")
        logger.debug(
            dict(
                msg="初始化参数",
                API_DOMAIN=self.API_DOMAIN,
                APP_KEY=self.APP_KEY,
                CORPORATION_ID=self.CORPORATION_ID,
                BIZ_ID=self.BIZ_ID,
                SECRET_KEY=self.SECRET_KEY,
                REPORT_DOMAIN=self.REPORT_DOMAIN,
            )
        )

    def request(self, api_name, method="GET", **kwargs):
        url = f"https://{self.API_DOMAIN}/{api_name}"
        # 请求头里面携带以下三个字段：corporationId、timestamp(精确到秒)、sign。
        headers = kwargs.get("headers", {})
        headers["corporationId"] = self.CORPORATION_ID
        headers["timestamp"] = str(int(datetime.datetime.now().timestamp()))
        headers["sign"] = get_sign(headers["timestamp"])
        kwargs["headers"] = headers
        params = kwargs.get("params", {})
        params["bizId"] = self.BIZ_ID
        kwargs["params"] = params

        logger.debug(
            dict(
                msg="请求参数",
                url=url,
                method=method,
                headers=headers,
                params=params,
                data=kwargs.get("data", {}),
                json=kwargs.get("json", {}),
            )
        )
        response = requests.request(method, url, **kwargs)
        if response.status_code != 200:
            logger.error(
                dict(
                    msg="请求失败",
                    status_code=response.status_code,
                    response=response.text,
                )
            )
            response.raise_for_status()
        response_data = response.json()
        return self.response(response_data)

    def response(self, data):
        logger.debug(dict(msg="返回值", response=data))
        code = data.get("code", -1)
        requestId = data.get("requestId", "")
        msg = data.get("message", "")
        if code != 0:
            logger.error(
                dict(
                    msg="接口返回错误",
                    code=code,
                    message=msg,
                    response=data,
                    requestId=requestId,
                )
            )
            raise Exception(f"接口返回错误，code={code}, message={msg}")
        return data



from .base import base


class User(base):
    def create(self, data):
        """
        创建用户
        https://tmc.qidian.qq.com/base/console/doc/13745?version=20250714

        创建单条客户数据
        {
            "identity":[ //客户身份
                {
                    "identityType":2, //客户身份类型
                    "identityValue":"18412301799" //客户身份值
                }
              ],
              "property":{ //客户属性
                  "name":"测试2222",
                  "arrayField": ["ab","cd"] //字符串数组类型示意
               }
        }
        """
        api_name = "cdp-entity/user/create"
        return self.request(
            api_name=api_name,
            json=data,
            method="POST",
        )

    def batch_create(self, data):
        """
        客户批量创建
        https://tmc.qidian.qq.com/base/console/doc/13746?version=20250714
        创建单条客户数据
        {
            "contents":[
                {
                    "identity":[
                        {
                            "identityType":2, //客户身份类型
                            "identityValue":"18412301799" //客户身份值
                            }
                    ],
                    "property":{ //客户属性
                        "属性字段Key":"属性值",
                        "arrayField": ["ab","cd"] //字符串数组类型示意
                    }
                }
            ]
        }
        """
        api_name = "cdp-entity/user/batchCreate"
        return self.request(
            api_name=api_name,
            json=data,
            method="POST",
        )

    def batch_edit(self, data):
        """
        客户批量更新
        https://tmc.qidian.qq.com/base/console/doc/13747?version=20250714

        请求示例：
        {
            "contents":[
                {
                    "id":"10283722" // 系统生成的统一客户OneID
                    "userIdentity": // 发布版本>=6.5
                        {
                            "identityType": 2,
                            "identityValue": "13428532631"
                        }
                    ,
                    "identity":[
                        {
                            "identityType":2,
                            "identityValue":"13601781755"
                        },
                        {
                            "identityType":1,
                            "identityValue":"18273631"
                        }
                    ],
                    "property":{
                        "客户属性字段Key":"客户属性值"
                        "arrayField": ["ab","cd"] //字符串数组类型示意
                    }
                }
            ]
        }
        """
        api_name = "cdp-entity/user/batchEdit"
        return self.request(
            api_name=api_name,
            json=data,
            method="POST",
        )

    def batch_delete(self, ids: list):
        """
        客户批量删除
        https://tmc.qidian.qq.com/base/console/doc/13748?version=20250714

        请求示例：
        {
            "ids":["66"] // 客户OneID列表
        }
        """
        api_name = "cdp-entity/user/batchDelete"
        return self.request(
            api_name=api_name,
            json={"ids": ids},
            method="POST",
        )

    def bind(self, contents: list):
        """
        客户身份绑定
        https://tmc.qidian.qq.com/base/console/doc/13749?version=20250714
        请求示例：
        {
            "contents": [
                {
                    "id": "10283722", // 系统生成的统一客户OneID
                    "identity":[ //客户身份
                        {
                            "identityType":2, //客户身份类型
                            "identityValue":"18412301799" //客户身份值
                        }
                    ]
                }
            ]
        }
        """
        api_name = "cdp-entity/user/identity/bind"
        return self.request(
            api_name=api_name,
            json={"contents": contents},
            method="POST",
        )

    def unbind(self, contents: list):
        """
        客户身份解绑
        https://tmc.qidian.qq.com/base/console/doc/14135?version=20250714
        请求示例：
        {
            "contents": [
                {
                    "id": "10283722", // 系统生成的统一客户OneID
                    "identity":[ //客户身份
                        {
                            "identityType":2, //客户身份类型
                            "identityValue":"18412301799" //客户身份值
                        }
                    ]
                }
            ]
        }
        """
        api_name = "cdp-entity/user/identity/unbind"
        return self.request(
            api_name=api_name,
            json={"contents": contents},
            method="POST",
        )

    def reset(self, contents: list):
        """
        客户身份重置
        https://tmc.qidian.qq.com/base/console/doc/14136?version=20250714
        请求示例：
        {
            "contents": [
                {
                    "id": "10283722", // 系统生成的统一客户OneID
                    "identity":[ //客户身份
                        {
                            "identityType":2, //客户身份类型
                            "identityValue":"18412301799" //客户身份值
                        }
                    ]
                }
            ]
        }
        """
        api_name = "cdp-entity/user/identity/reset"
        return self.request(
            api_name=api_name,
            json={"contents": contents},
            method="POST",
        )

    def queryList(self, pageNum: int = 1, pageSize: int = 10, queryCriteria: dict = {}):
        """
        客户列表查询
        https://tmc.qidian.qq.com/base/console/doc/13751?version=20250714
        请求示例：
        {
            "pageNum":1,
            "pageSize":10,
            "queryCriteria":{
                "filter": [
                    {
                        "property":"id",
                        "operation":"EQUAL",
                        "value":"67"
                    }
                ]
            }
        }
        查询条件没有filter时，表示查询所有客户； filter 之间默认是 and 关系；
        """
        api_name = "cdp-entity/user/queryList"
        queryCriteria = queryCriteria or {"filter": []}
        return self.request(
            api_name=api_name,
            json={
                "pageNum": pageNum,
                "pageSize": pageSize,
                "queryCriteria": queryCriteria,
            },
            method="POST",
        )

    def queryDetail(
        self, pageNum: int, pageSize: int, node: list = ["property"], items: list = []
    ):
        """
        客户详情查询
        基于不同类型的客户身份（客户OneID或其余客户身份，但只支持传某一种身份）查询客户详情（含客户属性、客户身份、客户标签），批量处理下单次上限100条。
        https://tmc.qidian.qq.com/base/console/doc/13752?version=20250714
        请求示例：
        {
            "pageNum":1,
            "pageSize":10,
            "node":["property"] //只返回property节点，指向返回节点
            "items":[
                {//只传saKey
                    "saKey" : "phone"
                    "saValue": "字段"
                },
                //只传oneId
                {
                    "oneId":"10283722" // 系统生成的统一客户OneID
                }
            ]
        }
        """
        api_name = "cdp-entity/user/queryDetail"
        return self.request(
            api_name=api_name,
            json={
                "pageNum": pageNum,
                "pageSize": pageSize,
                "node": node,
                "items": items,
            },
            method="POST",
        )



"""
@Author: Rqk
@Date: 2021-05-22
@Description:
"""

from setuptools import setup, find_packages

version = "1.0.2"
setup(
    name="pyqidian",
    version=version,
    keywords=[
        "qidian",
        "qidian_api",
    ],
    description="",
    long_description="",
    license="MIT Licence",
    url="https://github.com/renqiukai/pyqidian",
    author="Renqiukai",
    author_email="renqiukai@qq.com",
    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    install_requires=["requests", "loguru"],
)
